# ncgmp09_create.py
#   Python script to create an empty NCGMP09-style
#   ArcGIS 9.3 geodatabase for geologic map data
#
#   Ralph Haugerud, USGS
#
#   Takes 2 arguments: <outputDir>  <geoDataBaseName> 
#     to use local directory, use # for outputDir
#     if geoDatabaseName ends in .gdb, a file geodatabase will be created
#     if geoDatabaseName ends in .mdb, a personal geodatabase will be created
#
#   Requires that ncgmp09_definition.py be present in the local directory 
#     or in the appropriate Python library directory.
#
#   On my laptop, this takes many minutes to run and creates a 
#      ~0.5GB file/directory
#   This version of code traps no errors. No checking for existence of outputDir
#      or nonexistence of geoDataBaseName
#
# To use this:
#   	1) run script, e.g.
#		C:\myworkspace> ncgmp09_create.py # newgeodatabase.gdb
#	2) Open newgeodatabase in ArcCatalog, doubleclick on feature data set GeologicMap,
#	   and assign a spatial reference system
#	3) If there will be multiple cross-sections, copy feature data set CrossSectionA, 
#	   paste, and rename (CrossSectionB, etc.). You should also rename the constituent
# 	   feature classes and, within each feature class, field <featureClass>_ID
#	4) If you use the CorrelationOfMapUnits feature data set, note that you will have to 
#	   manually create the annotation feature class CMUText and add field ParagraphStyle.
#	   (I haven't yet found a way to script the creation of an annotation feature class.)
#	5) If there will be non-standard point feature classes (photos, mineral occurrences,
#	   etc.), copy/paste/rename feature class GenericPoint or GenericSample, as appropriate,
#	   and add necessary fields to new feature class
#	6) Delete any unneeded feature classes and feature data sets
#	7) Load data. Edit as needed

import arcgisscripting, sys, os
from ncgmp09_definition import tableDict

versionString = 'ncgmp09_create.py, version of 18 April 2009'

default = '#'

transDict =     { 'String': 'TEXT',
			'Single': 'FLOAT',
			'Double': 'DOUBLE',
			'NoNulls':'NON_NULLABLE',
			'NullsOK':'NULLABLE',
			'Date'  : 'DATE'  }

def usage():
	print("""Usage:
systemprompt> ncgmp09_create.py <directory> <geodatabaseName> 
   <directory> is existing directory in which new geodatabaseName is to 
      be created, use # for current directory
   <geodatabaseName> is name of gdb to be created, with extension
      .gdb causes a file geodatabase to be created
      .mdb causes a personal geodatabase to be created

  Then, in ArcCatalog:
  * Open newgeodatabase, doubleclick on feature data set GeologicMap,
    and assign a spatial reference system.
  * If there will be multiple cross-sections, copy feature data set CrossSectionA, 
    paste, and rename (CrossSectionB, etc.). You should also rename the constituent
    feature classes and, within each feature class, field <featureClass>_ID.
  * If you use the CorrelationOfMapUnits feature data set, note that you will have to 
    manually create the annotation feature class CMUText and add field ParagraphStyle.
    (I haven't yet found a way to script the creation of an annotation feature class.)
  * If there will be non-standard point feature classes (photos, mineral occurrences,
    etc.), copy/paste/rename feature class GenericPoint or GenericSample, as appropriate,
    and add necessary fields to new feature class.
  * Delete any unneeded feature classes and feature data sets.
  * Load data, if data already are in GIS form. 
  Edit data as needed.

""")

def createFeatureClass(thisDB,featureDataSet,featureClass,shapeType,fieldDefs):
	gp.CreateFeatureClass_Management(thisDB+'/'+featureDataSet,featureClass,shapeType)
	thisFC = thisDB+'/'+featureDataSet+'/'+featureClass
	for fDef in fieldDefs:
		if fDef[1] == 'String':
			gp.AddField_Management(thisFC,fDef[0],transDict[fDef[1]],default,default,fDef[3],default,transDict[fDef[2]])
		else:
			gp.AddField_Management(thisFC,fDef[0],transDict[fDef[1]],default,default,default,default,transDict[fDef[2]])

def main():
	gp = arcgisscripting.create(9.3)
	outputDir = sys.argv[1]
	thisDB = sys.argv[2]

	if outputDir == default:
		outputDir = os.getcwd()
	
	# create personal gdb in local directory
	if thisDB[-4:] == '.mdb':
		gp.createPersonalGDB_management(outputDir,thisDB)
	if thisDB[-4:] == '.gdb':
		gp.createFileGDB_management(outputDir,thisDB)
		versionFile = open(thisDB+'/00readme.txt','write')
		versionFile.write('Geodatabase created by '+versionString+'\n')
		versionFile.close()
	thisDB = outputDir+'/'+thisDB
	gp.refreshcatalog(thisDB)

	# create tables
	tables = ['DescriptionOfMapUnits','StandardLithology','DataSources',
      	    'ExtendedAttributes','GeologicEvents','Glossary']
	for table in tables:
		gp.CreateTable(thisDB,table)
		fieldDefs = tableDict[table]
		for fDef in fieldDefs:
			if fDef[1] == 'String':
				gp.AddField_Management(thisDB+'/'+table,fDef[0],transDict[fDef[1]],default,default,fDef[3],default,transDict[fDef[2]])
			else:
				gp.AddField_Management(thisDB+'/'+table,fDef[0],transDict[fDef[1]],default,default,default,default,transDict[fDef[2]])
	# create feature dataset GeologicMap
	gp.CreateFeatureDataset_management(thisDB,'GeologicMap')
	# create feature classes in GeologicMap
	# poly feature classes
	featureClasses = ['MapUnitPolys','OverlayPolys']
	for featureClass in featureClasses:
		fieldDefs = tableDict[featureClass]
		createFeatureClass(thisDB,'GeologicMap',featureClass,'POLYGON',fieldDefs)
	# line feature classes
	featureClasses = ['ContactsAndFaults','ConcealedContactsAndFaults','OtherLines']
	for featureClass in featureClasses:
		fieldDefs = tableDict[featureClass]
		createFeatureClass(thisDB,'GeologicMap',featureClass,'POLYLINE',fieldDefs)
	# point feature classes
	featureClasses = ['OrientationPoints','GeochronPoints','FossilPoints','MapUnitPoints','Stations',
		      'GenericSamples','GenericPoints']
	for featureClass in featureClasses:
		if featureClass == 'MapUnitPoints': 
			fieldDefs = tableDict['MapUnitPolys']
		else:	
			fieldDefs = tableDict[featureClass]
		createFeatureClass(thisDB,'GeologicMap',featureClass,'POINT',fieldDefs)

	# create feature dataset CorrelationOfMapUnits
	gp.CreateFeatureDataset_management(thisDB,'CorrelationOfMapUnits')
	fieldDefs = tableDict['CMUPolys']
	createFeatureClass(thisDB,'CorrelationOfMapUnits','CMUPolys','POLYGON',fieldDefs)
	fieldDefs = tableDict['CMULines']
	createFeatureClass(thisDB,'CorrelationOfMapUnits','CMULines','POLYLINE',fieldDefs)
	fieldDefs = tableDict['CMUPoints']
	createFeatureClass(thisDB,'CorrelationOfMapUnits','CMUPoints','POINT',fieldDefs)

	#create feature dataset CrossSectionA
	gp.CreateFeatureDataset_management(thisDB,'CrossSectionA')
	fieldDefs = tableDict['MapUnitPolys']
	fieldDefs[0][0] = 'CSAMapUnitPolys_ID'
	createFeatureClass(thisDB,'CrossSectionA','CSAMapUnitPolys','POLYGON',fieldDefs)
	fieldDefs = tableDict['ContactsAndFaults']
	fieldDefs[0][0] = 'CSAContactsAndFaults_ID'
	createFeatureClass(thisDB,'CrossSectionA','CSAContactsAndFaults','POLYLINE',fieldDefs)
	fieldDefs = tableDict['OrientationPoints']
	fieldDefs[0][0] = 'CSAOrientationPoints_ID'
	createFeatureClass(thisDB,'CrossSectionA','CSAOrientationPoints','POINT',fieldDefs)


try:
	if len(sys.argv) == 3:
		main()
	else:
		usage()
except:
	#usage()
	thisX = True
